@@ -29,19 +29,10 @@ abstract class AbstractNode implements NodeInterface, LoggerAwareInterface
 {
     use LoggerAwareTrait;
 
-    /**
-     * Instance of the node factory to create sub elements, container and single element expansions.
-     *
-     * @var NodeFactory
-     */
-    protected $nodeFactory;
-
     /**
      * Main data array to work on, given from parent to child elements
-     *
-     * @var array
      */
-    protected $data = [];
+    protected array $data = [];
 
     /**
      * A list of default field information added to the element / container.
@@ -67,15 +58,11 @@ abstract class AbstractNode implements NodeInterface, LoggerAwareInterface
     protected $defaultFieldWizard = [];
 
     /**
-     * Set data to data array and register node factory to render sub elements
-     *
-     * @param NodeFactory $nodeFactory
-     * @param array $data
+     * Retrieve the current data array from NodeFactory.
      */
-    public function __construct(NodeFactory $nodeFactory, array $data)
+    public function setData(array $data): void
     {
         $this->data = $data;
-        $this->nodeFactory = $nodeFactory;
     }
 
     /**
@@ -83,26 +70,25 @@ abstract class AbstractNode implements NodeInterface, LoggerAwareInterface
      *
      * @return array As defined in initializeResultArray() of AbstractNode
      */
-    abstract public function render();
+    abstract public function render(): array;
 
     /**
      * Initialize the array that is returned to parent after calling. This structure
      * is identical for *all* nodes. Parent will merge the return of a child with its
      * own stuff and in itself return an array of the same structure.
-     *
-     * @return array
      */
     protected function initializeResultArray(): array
     {
+        /** @var list<\TYPO3\CMS\Core\Page\JavaScriptModuleInstruction> */
+        $javaScriptModules = [];
+
         return [
-            'additionalJavaScriptPost' => [],
+            // @todo deprecate additionalHiddenFields in TYPO3 v12.0 - this return key is essentially
+            //       useless. Elements can simply submit their hidden HTML fields in the html key.
             'additionalHiddenFields' => [],
             'additionalInlineLanguageLabelFiles' => [],
             'stylesheetFiles' => [],
-            // can hold strings or arrays,
-            // string = requireJS module,
-            // array = requireJS module + callback e.g. array('TYPO3/Foo/Bar', 'function() {}')
-            'requireJsModules' => [],
+            'javaScriptModules' => $javaScriptModules,
             'inlineData' => [],
             'html' => '',
         ];
@@ -123,17 +109,14 @@ abstract class AbstractNode implements NodeInterface, LoggerAwareInterface
         if ($mergeHtml && !empty($childReturn['html'])) {
             $existing['html'] .= LF . $childReturn['html'];
         }
-        foreach ($childReturn['additionalJavaScriptPost'] ?? [] as $value) {
-            $existing['additionalJavaScriptPost'][] = $value;
-        }
         foreach ($childReturn['additionalHiddenFields'] ?? [] as $value) {
             $existing['additionalHiddenFields'][] = $value;
         }
         foreach ($childReturn['stylesheetFiles'] ?? [] as $value) {
             $existing['stylesheetFiles'][] = $value;
         }
-        foreach ($childReturn['requireJsModules'] ?? [] as $module) {
-            $existing['requireJsModules'][] = $module;
+        foreach ($childReturn['javaScriptModules'] ?? [] as $module) {
+            $existing['javaScriptModules'][] = $module;
         }
         foreach ($childReturn['additionalInlineLanguageLabelFiles'] ?? [] as $inlineLanguageLabelFile) {
             $existing['additionalInlineLanguageLabelFiles'][] = $inlineLanguageLabelFile;
@@ -149,15 +132,12 @@ abstract class AbstractNode implements NodeInterface, LoggerAwareInterface
 
     /**
      * Build JSON string for validations rules.
-     *
-     * @param array $config
-     * @return string
      */
     protected function getValidationDataAsJsonString(array $config): string
     {
         $validationRules = [];
         if (!empty($config['eval'])) {
-            $evalList = GeneralUtility::trimExplode(',', $config['eval'], true);
+            $evalList = GeneralUtility::trimExplode(',', $config['eval'] ?? '', true);
             foreach ($evalList as $evalType) {
                 $validationRules[] = [
                     'type' => $evalType,
@@ -168,11 +148,21 @@ abstract class AbstractNode implements NodeInterface, LoggerAwareInterface
             $newValidationRule = [
                 'type' => 'range',
             ];
+
+            $isDateTime = ($config['type'] ?? '') === 'datetime';
             if (!empty($config['range']['lower'])) {
-                $newValidationRule['lower'] = $config['range']['lower'];
+                $lower = (int)$config['range']['lower'];
+                if ($isDateTime) {
+                    $lower = gmdate('c', $lower + (int)(date('Z', $lower)));
+                }
+                $newValidationRule['lower'] = $lower;
             }
             if (!empty($config['range']['upper'])) {
-                $newValidationRule['upper'] = $config['range']['upper'];
+                $upper = (int)$config['range']['upper'];
+                if ($isDateTime) {
+                    $upper = gmdate('c', $upper + (int)(date('Z', $upper)));
+                }
+                $newValidationRule['upper'] = $upper;
             }
             $validationRules[] = $newValidationRule;
         }
@@ -183,12 +173,15 @@ abstract class AbstractNode implements NodeInterface, LoggerAwareInterface
             $validationRules[] = [
                 'type' => $type,
                 'minItems' => $minItems,
-                'maxItems' => $maxItems
+                'maxItems' => $maxItems,
             ];
         }
         if (!empty($config['required'])) {
             $validationRules[] = ['type' => 'required'];
         }
+        if (!empty($config['min'])) {
+            $validationRules[] = ['type' => 'min'];
+        }
         return json_encode($validationRules);
     }
 }
