@@ -18,7 +18,7 @@ namespace TYPO3\CMS\Backend\Template\Components\Buttons;
 /**
  * Interface for buttons
  */
-interface ButtonInterface
+interface ButtonInterface extends \Stringable
 {
     /**
      * Validates all set parameters of a button.
@@ -34,13 +34,6 @@ interface ButtonInterface
      */
     public function getType();
 
-    /**
-     * RenderMethod of a button if accessed as string from fluid
-     *
-     * @return string
-     */
-    public function __toString();
-
     /**
      * Renders the markup for the button
      *
