@@ -17,44 +17,40 @@ declare(strict_types=1);
 
 namespace TYPO3\CMS\Core\Controller;
 
+use Psr\EventDispatcher\EventDispatcherInterface;
+use Psr\Http\Message\ResponseFactoryInterface;
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
-use TYPO3\CMS\Core\Http\Response;
+use Symfony\Component\DependencyInjection\Attribute\Autoconfigure;
+use TYPO3\CMS\Core\Crypto\HashService;
 use TYPO3\CMS\Core\Imaging\ImageManipulation\CropVariantCollection;
-use TYPO3\CMS\Core\Resource\Exception\FileDoesNotExistException;
+use TYPO3\CMS\Core\Resource\Event\ModifyFileDumpEvent;
 use TYPO3\CMS\Core\Resource\File;
 use TYPO3\CMS\Core\Resource\FileInterface;
 use TYPO3\CMS\Core\Resource\FileReference;
-use TYPO3\CMS\Core\Resource\Hook\FileDumpEIDHookInterface;
 use TYPO3\CMS\Core\Resource\ProcessedFile;
 use TYPO3\CMS\Core\Resource\ProcessedFileRepository;
 use TYPO3\CMS\Core\Resource\ResourceFactory;
+use TYPO3\CMS\Core\Resource\ResourceInterface;
 use TYPO3\CMS\Core\Resource\Security\FileNameValidator;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
+use TYPO3\CMS\Core\Utility\PathUtility;
 
-/**
- * Class FileDumpController
- */
-class FileDumpController
+#[Autoconfigure(public: true)]
+readonly class FileDumpController
 {
-    /**
-     * @var ResourceFactory
-     */
-    protected $resourceFactory;
-
-    public function __construct(ResourceFactory $resourceFactory)
-    {
-        $this->resourceFactory = $resourceFactory;
-    }
+    public function __construct(
+        protected EventDispatcherInterface $eventDispatcher,
+        protected ResourceFactory $resourceFactory,
+        protected ResponseFactoryInterface $responseFactory,
+        protected HashService $hashService
+    ) {}
 
     /**
      * Main method to dump a file
      *
-     * @param ServerRequestInterface $request
-     * @return ResponseInterface
      * @throws \InvalidArgumentException
      * @throws \RuntimeException
-     * @throws FileDoesNotExistException
      * @throws \UnexpectedValueException
      */
     public function dumpAction(ServerRequestInterface $request): ResponseInterface
@@ -62,24 +58,21 @@ class FileDumpController
         $parameters = $this->buildParametersFromRequest($request);
 
         if (!$this->isTokenValid($parameters, $request)) {
-            return (new Response())->withStatus(403);
+            return $this->responseFactory->createResponse(403);
         }
         $file = $this->createFileObjectByParameters($parameters);
         if ($file === null) {
-            return (new Response())->withStatus(404);
+            return $this->responseFactory->createResponse(404);
         }
 
-        // Hook: allow some other process to do some security/access checks. Hook should return 403 response if access is rejected, void otherwise
-        foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['FileDumpEID.php']['checkFileAccess'] ?? [] as $className) {
-            $hookObject = GeneralUtility::makeInstance($className);
-            if (!$hookObject instanceof FileDumpEIDHookInterface) {
-                throw new \UnexpectedValueException($className . ' must implement interface ' . FileDumpEIDHookInterface::class, 1394442417);
-            }
-            $response = $hookObject->checkFileAccess($file);
-            if ($response instanceof ResponseInterface) {
-                return $response;
-            }
+        // Allow some other process to do some security/access checks.
+        // Event Listeners should return a 403 response if access is rejected
+        $event = new ModifyFileDumpEvent($file, $request);
+        $event = $this->eventDispatcher->dispatch($event);
+        if ($event->isPropagationStopped()) {
+            return $this->applyContentSecurityPolicy($event->getFile(), $event->getResponse());
         }
+        $file = $event->getFile();
 
         $processingInstructions = [];
 
@@ -107,7 +100,7 @@ class FileDumpController
                     'minWidth' => $size[2] ? (int)$size[2] : null,
                     'minHeight' => $size[3] ? (int)$size[3] : null,
                     'maxWidth' => $size[4] ? (int)$size[4] : null,
-                    'maxHeight' => $size[5] ? (int)$size[5] : null
+                    'maxHeight' => $size[5] ? (int)$size[5] : null,
                 ]
             );
         }
@@ -120,7 +113,14 @@ class FileDumpController
             $file = $file->process(ProcessedFile::CONTEXT_IMAGECROPSCALEMASK, $processingInstructions);
         }
 
-        return $file->getStorage()->streamFile($file);
+        return $this->applyContentSecurityPolicy(
+            $file,
+            $file->getStorage()->streamFile(
+                $file,
+                (bool)($parameters['dl'] ?? false),
+                $parameters['fn'] ?? null
+            )
+        );
     }
 
     protected function buildParametersFromRequest(ServerRequestInterface $request): array
@@ -154,9 +154,19 @@ class FileDumpController
             $parameters['s'] = $s;
         }
         // File's crop variant
-        $v = (string)($queryParams['cv'] ?? '');
-        if ($v) {
-            $parameters['cv'] = (string)$v;
+        $cv = (string)($queryParams['cv'] ?? '');
+        if ($cv) {
+            $parameters['cv'] = $cv;
+        }
+        // As download
+        $dl = (string)($queryParams['dl'] ?? '');
+        if ($dl) {
+            $parameters['dl'] = (int)$dl;
+        }
+        // Alternative file name
+        $fn = (string)($queryParams['fn'] ?? '');
+        if ($fn) {
+            $parameters['fn'] = $fn;
         }
 
         return $parameters;
@@ -165,13 +175,12 @@ class FileDumpController
     protected function isTokenValid(array $parameters, ServerRequestInterface $request): bool
     {
         return hash_equals(
-            GeneralUtility::hmac(implode('|', $parameters), 'resourceStorageDumpFile'),
+            $this->hashService->hmac(implode('|', $parameters), 'resourceStorageDumpFile'),
             $request->getQueryParams()['token'] ?? ''
         );
     }
 
     /**
-     * @param array $parameters
      * @return File|FileReference|ProcessedFile|null
      */
     protected function createFileObjectByParameters(array $parameters)
@@ -198,12 +207,11 @@ class FileDumpController
         } elseif (isset($parameters['p'])) {
             try {
                 $processedFileRepository = GeneralUtility::makeInstance(ProcessedFileRepository::class);
-                /** @var ProcessedFile|null $file */
-                $file = $processedFileRepository->findByUid($parameters['p']);
-                if (!$file || $file->isDeleted() || !$this->isFileValid($file->getOriginalFile())) {
+                $file = $processedFileRepository->findByUid((int)$parameters['p']);
+                if ($file->isDeleted() || !$this->isFileValid($file->getOriginalFile())) {
                     $file = null;
                 }
-            } catch (\Exception $e) {
+            } catch (\Exception) {
                 $file = null;
             }
         }
@@ -216,4 +224,21 @@ class FileDumpController
             || GeneralUtility::makeInstance(FileNameValidator::class)
                 ->isValid(basename($file->getIdentifier()));
     }
+
+    /**
+     * Applies hard-coded content-security-policy (CSP) for file to be dumped.
+     */
+    protected function applyContentSecurityPolicy(ResourceInterface $file, ResponseInterface $response): ResponseInterface
+    {
+        $extension = PathUtility::pathinfo($file->getName(), PATHINFO_EXTENSION);
+        // same as in `typo3/sysext/install/Resources/Private/FolderStructureTemplateFiles/resources-root-htaccess`
+        if ($extension === 'pdf' || $response->getHeaderLine('content-type') === 'application/pdf') {
+            $policy = "default-src 'self' 'unsafe-inline'; script-src 'none'; object-src 'self'; plugin-types application/pdf;";
+        } elseif ($extension === 'svg' || $response->getHeaderLine('content-type') === 'image/svg+xml') {
+            $policy = "default-src 'self'; script-src 'none'; style-src 'unsafe-inline'; object-src 'none';";
+        } else {
+            $policy = "default-src 'self'; script-src 'none'; style-src 'none'; object-src 'none';";
+        }
+        return $response->withAddedHeader('content-security-policy', $policy);
+    }
 }
