@@ -22,21 +22,7 @@ namespace TYPO3\CMS\RteCKEditor\Form\Element\Event;
  */
 final class BeforePrepareConfigurationForEditorEvent
 {
-    /**
-     * @var array
-     */
-    private $configuration;
-
-    /**
-     * @var array
-     */
-    private $data;
-
-    public function __construct(array $configuration, array $data)
-    {
-        $this->configuration = $configuration;
-        $this->data = $data;
-    }
+    public function __construct(private $configuration, private readonly array $data) {}
 
     public function getData(): array
     {
